@@ -31,6 +31,8 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <unistd.h>
 #include <sys/stat.h>
 
+#include <fitsio.h>
+
 #include <gnuastro/checkset.h>
 
 
@@ -44,13 +46,13 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #define NOTEMSG_SMALLERONE  "should be >=0 and <=1"
 
 
-#define CHECKFULLNUMBER   if(*tailptr!='\0'){				\
-    if(filename)							\
-      error_at_line(EXIT_FAILURE, 0, filename, lineno,			\
-		    FIXEDFORFILE" "NOTEMSG_NOTNUMBER, lo, optarg);	\
-    else								\
-      error(EXIT_FAILURE, 0, FIXEDFOROPTION" "NOTEMSG_NOTNUMBER,	\
-	    lo, so, optarg);						\
+#define CHECKFULLNUMBER   if(*tailptr!='\0'){                           \
+    if(filename)                                                        \
+      error_at_line(EXIT_FAILURE, 0, filename, lineno,                  \
+                    FIXEDFORFILE" "NOTEMSG_NOTNUMBER, lo, optarg);      \
+    else                                                                \
+      error(EXIT_FAILURE, 0, FIXEDFOROPTION" "NOTEMSG_NOTNUMBER,        \
+            lo, so, optarg);                                            \
   }
 
 
@@ -62,7 +64,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 /* See if a given string is a floating point number, if so, put it in
    the number.*/
 int
-strisdouble(char *string, double *out)
+gal_checkset_str_is_double(char *string, double *out)
 {
   char *tailptr;
   double tmp=*out;
@@ -112,11 +114,11 @@ gal_checkset_int_zero_or_one(char *optarg, int *var, char *lo, char so,
   if(tmp!=0 && tmp!=1)
     {
       if(filename)
-	error_at_line(EXIT_FAILURE, 0, filename, lineno,
-		      FIXEDFORFILE" "NOTEMSG_EQZEROORONE, lo, optarg);
+        error_at_line(EXIT_FAILURE, 0, filename, lineno,
+                      FIXEDFORFILE" "NOTEMSG_EQZEROORONE, lo, optarg);
       else
-	error(EXIT_FAILURE, 0, FIXEDFOROPTION" "NOTEMSG_EQZEROORONE,
-	      lo, so, optarg);
+        error(EXIT_FAILURE, 0, FIXEDFOROPTION" "NOTEMSG_EQZEROORONE,
+              lo, so, optarg);
     }
 }
 
@@ -125,8 +127,8 @@ gal_checkset_int_zero_or_one(char *optarg, int *var, char *lo, char so,
 
 
 void
-gal_checkset_int_4_or_8(char *optarg, int *var, char *lo, char so, char *spack,
-                        char *filename, size_t lineno)
+gal_checkset_int_4_or_8(char *optarg, int *var, char *lo, char so,
+                        char *spack, char *filename, size_t lineno)
 {
   long tmp;
   char *tailptr;
@@ -137,11 +139,11 @@ gal_checkset_int_4_or_8(char *optarg, int *var, char *lo, char so, char *spack,
   if(tmp!=4 && tmp!=8)
     {
       if(filename)
-	error_at_line(EXIT_FAILURE, 0, filename, lineno,
-		      FIXEDFORFILE" "NOTEMSG_EQZEROORONE, lo, optarg);
+        error_at_line(EXIT_FAILURE, 0, filename, lineno,
+                      FIXEDFORFILE" "NOTEMSG_EQZEROORONE, lo, optarg);
       else
-	error(EXIT_FAILURE, 0, FIXEDFOROPTION" "NOTEMSG_EQZEROORONE,
-	      lo, so, optarg);
+        error(EXIT_FAILURE, 0, FIXEDFOROPTION" "NOTEMSG_EQZEROORONE,
+              lo, so, optarg);
     }
 }
 
@@ -150,8 +152,8 @@ gal_checkset_int_4_or_8(char *optarg, int *var, char *lo, char so, char *spack,
 
 
 void
-gal_checkset_int_el_zero(char *optarg, int *var, char *lo, char so, char *spack,
-                         char *filename, size_t lineno)
+gal_checkset_int_el_zero(char *optarg, int *var, char *lo, char so,
+                         char *spack, char *filename, size_t lineno)
 {
   long tmp;
   char *tailptr;
@@ -161,11 +163,11 @@ gal_checkset_int_el_zero(char *optarg, int *var, char *lo, char so, char *spack,
   if(tmp<0)
     {
       if(filename)
-	error_at_line(EXIT_FAILURE, 0, filename, lineno,
-		      FIXEDFORFILE" "NOTEMSG_LEQZERO, lo, optarg);
+        error_at_line(EXIT_FAILURE, 0, filename, lineno,
+                      FIXEDFORFILE" "NOTEMSG_LEQZERO, lo, optarg);
       else
-	error(EXIT_FAILURE, 0, FIXEDFOROPTION" "NOTEMSG_LEQZERO,
-	      lo, so, optarg);
+        error(EXIT_FAILURE, 0, FIXEDFOROPTION" "NOTEMSG_LEQZERO,
+              lo, so, optarg);
     }
 }
 
@@ -174,8 +176,8 @@ gal_checkset_int_el_zero(char *optarg, int *var, char *lo, char so, char *spack,
 
 
 void
-gal_checkset_int_l_zero(char *optarg, int *var, char *lo, char so, char *spack,
-                        char *filename, size_t lineno)
+gal_checkset_int_l_zero(char *optarg, int *var, char *lo, char so,
+                        char *spack, char *filename, size_t lineno)
 {
   long tmp;
   char *tailptr;
@@ -185,11 +187,11 @@ gal_checkset_int_l_zero(char *optarg, int *var, char *lo, char so, char *spack,
   if(tmp<=0)
     {
       if(filename)
-	error_at_line(EXIT_FAILURE, 0, filename, lineno,
-		      FIXEDFORFILE" "NOTEMSG_LARGERZERO, lo, optarg);
+        error_at_line(EXIT_FAILURE, 0, filename, lineno,
+                      FIXEDFORFILE" "NOTEMSG_LARGERZERO, lo, optarg);
       else
-	error(EXIT_FAILURE, 0, FIXEDFOROPTION" "NOTEMSG_LARGERZERO,
-	      lo, so, optarg);
+        error(EXIT_FAILURE, 0, FIXEDFOROPTION" "NOTEMSG_LARGERZERO,
+              lo, so, optarg);
     }
 }
 
@@ -210,11 +212,11 @@ gal_checkset_int_smaller_equal_to(char *optarg, int *var, char *lo, char so,
   if(tmp>maxvalue)
     {
       if(filename)
-	error_at_line(EXIT_FAILURE, 0, filename, lineno,
-		      FIXEDFORFILE" "NOTEMSG_4OR8, lo, optarg);
+        error_at_line(EXIT_FAILURE, 0, filename, lineno,
+                      FIXEDFORFILE" "NOTEMSG_4OR8, lo, optarg);
       else
-	error(EXIT_FAILURE, 0, FIXEDFOROPTION" "NOTEMSG_4OR8,
-	      lo, so, optarg);
+        error(EXIT_FAILURE, 0, FIXEDFOROPTION" "NOTEMSG_4OR8,
+              lo, so, optarg);
     }
 }
 
@@ -234,11 +236,11 @@ gal_checkset_long_el_zero(char *optarg, long *var, char *lo, char so,
   if(tmp<0)
     {
       if(filename)
-	error_at_line(EXIT_FAILURE, 0, filename, lineno,
-		      FIXEDFORFILE" "NOTEMSG_LEQZERO, lo, optarg);
+        error_at_line(EXIT_FAILURE, 0, filename, lineno,
+                      FIXEDFORFILE" "NOTEMSG_LEQZERO, lo, optarg);
       else
-	error(EXIT_FAILURE, 0, FIXEDFOROPTION" "NOTEMSG_LEQZERO,
-	      lo, so, optarg);
+        error(EXIT_FAILURE, 0, FIXEDFOROPTION" "NOTEMSG_LEQZERO,
+              lo, so, optarg);
     }
 }
 
@@ -247,8 +249,8 @@ gal_checkset_long_el_zero(char *optarg, long *var, char *lo, char so,
 
 
 void
-gal_checkset_any_long(char *optarg, long *var, char *lo, char so, char *spack,
-                      char *filename, size_t lineno)
+gal_checkset_any_long(char *optarg, long *var, char *lo, char so,
+                      char *spack, char *filename, size_t lineno)
 {
   char *tailptr;
   *var=strtol(optarg, &tailptr, 0);
@@ -277,11 +279,11 @@ gal_checkset_sizet_el_zero(char *optarg, size_t *var, char *lo, char so,
   if(tmp<0)
     {
       if(filename)
-	error_at_line(EXIT_FAILURE, 0, filename, lineno,
-		      FIXEDFORFILE" "NOTEMSG_LEQZERO, lo, optarg);
+        error_at_line(EXIT_FAILURE, 0, filename, lineno,
+                      FIXEDFORFILE" "NOTEMSG_LEQZERO, lo, optarg);
       else
-	error(EXIT_FAILURE, 0, FIXEDFOROPTION" "NOTEMSG_LEQZERO,
-	      lo, so, optarg);
+        error(EXIT_FAILURE, 0, FIXEDFOROPTION" "NOTEMSG_LEQZERO,
+              lo, so, optarg);
     }
 }
 
@@ -301,11 +303,11 @@ gal_checkset_sizet_l_zero(char *optarg, size_t *var, char *lo, char so,
   if(tmp<=0)
     {
       if(filename)
-	error_at_line(EXIT_FAILURE, 0, filename, lineno,
-		      FIXEDFORFILE" "NOTEMSG_LARGERZERO, lo, optarg);
+        error_at_line(EXIT_FAILURE, 0, filename, lineno,
+                      FIXEDFORFILE" "NOTEMSG_LARGERZERO, lo, optarg);
       else
-	error(EXIT_FAILURE, 0, FIXEDFOROPTION" "NOTEMSG_LARGERZERO,
-	      lo, so, optarg);
+        error(EXIT_FAILURE, 0, FIXEDFOROPTION" "NOTEMSG_LARGERZERO,
+              lo, so, optarg);
     }
 }
 
@@ -326,11 +328,11 @@ gal_checkset_sizet_p_odd(char *optarg, size_t *var, char *lo, char so,
   if(tmp<0 || tmp%2==0)
     {
       if(filename)
-	error_at_line(EXIT_FAILURE, 0, filename, lineno,
-		      FIXEDFORFILE" should be >0 and odd.", lo, optarg);
+        error_at_line(EXIT_FAILURE, 0, filename, lineno,
+                      FIXEDFORFILE" should be >0 and odd.", lo, optarg);
       else
-	error(EXIT_FAILURE, 0, FIXEDFOROPTION" should be >0 and odd",
-	      lo, so, optarg);
+        error(EXIT_FAILURE, 0, FIXEDFOROPTION" should be >0 and odd",
+              lo, so, optarg);
     }
 }
 
@@ -339,8 +341,8 @@ gal_checkset_sizet_p_odd(char *optarg, size_t *var, char *lo, char so,
 
 
 void
-gal_checkset_float_l_0(char *optarg, float *var, char *lo, char so, char* spack,
-                       char *filename, size_t lineno)
+gal_checkset_float_l_0(char *optarg, float *var, char *lo, char so,
+                       char* spack, char *filename, size_t lineno)
 {
   float tmp;
   char *tailptr;
@@ -350,11 +352,11 @@ gal_checkset_float_l_0(char *optarg, float *var, char *lo, char so, char* spack,
   if(tmp<=0)
     {
       if(filename)
-	error_at_line(EXIT_FAILURE, 0, filename, lineno,
-		      FIXEDFORFILE" "NOTEMSG_LARGERZERO, lo, optarg);
+        error_at_line(EXIT_FAILURE, 0, filename, lineno,
+                      FIXEDFORFILE" "NOTEMSG_LARGERZERO, lo, optarg);
       else
-	error(EXIT_FAILURE, 0, FIXEDFOROPTION" "NOTEMSG_LARGERZERO,
-	      lo, so, optarg);
+        error(EXIT_FAILURE, 0, FIXEDFOROPTION" "NOTEMSG_LARGERZERO,
+              lo, so, optarg);
     }
 }
 
@@ -374,11 +376,11 @@ gal_checkset_float_l_0_s_1(char *optarg, float *var, char *lo, char so,
   if(tmp>1.0f || tmp<0)
     {
       if(filename)
-	error_at_line(EXIT_FAILURE, 0, filename, lineno,
-		      FIXEDFORFILE" "NOTEMSG_SMALLERONE, lo, optarg);
+        error_at_line(EXIT_FAILURE, 0, filename, lineno,
+                      FIXEDFORFILE" "NOTEMSG_SMALLERONE, lo, optarg);
       else
-	error(EXIT_FAILURE, 0, FIXEDFOROPTION" "NOTEMSG_SMALLERONE,
-	      lo, so, optarg);
+        error(EXIT_FAILURE, 0, FIXEDFOROPTION" "NOTEMSG_SMALLERONE,
+              lo, so, optarg);
     }
 }
 
@@ -386,8 +388,8 @@ gal_checkset_float_l_0_s_1(char *optarg, float *var, char *lo, char so,
 
 
 void
-gal_checkset_any_float(char *optarg, float *var, char *lo, char so, char *spack,
-                       char *filename, size_t lineno)
+gal_checkset_any_float(char *optarg, float *var, char *lo, char so,
+                       char *spack, char *filename, size_t lineno)
 {
   char *tailptr;
   *var=strtof(optarg, &tailptr);
@@ -410,11 +412,11 @@ gal_checkset_double_l_0(char *optarg, double *var, char *lo, char so,
   if(tmp<=0)
     {
       if(filename)
-	error_at_line(EXIT_FAILURE, 0, filename, lineno,
-		      FIXEDFORFILE" "NOTEMSG_LARGERZERO, lo, optarg);
+        error_at_line(EXIT_FAILURE, 0, filename, lineno,
+                      FIXEDFORFILE" "NOTEMSG_LARGERZERO, lo, optarg);
       else
-	error(EXIT_FAILURE, 0, FIXEDFOROPTION" "NOTEMSG_LARGERZERO,
-	      lo, so, optarg);
+        error(EXIT_FAILURE, 0, FIXEDFOROPTION" "NOTEMSG_LARGERZERO,
+              lo, so, optarg);
     }
 }
 
@@ -434,11 +436,11 @@ gal_checkset_double_el_0(char *optarg, double *var, char *lo, char so,
   if(tmp<0)
     {
       if(filename)
-	error_at_line(EXIT_FAILURE, 0, filename, lineno,
-		      FIXEDFORFILE" "NOTEMSG_LARGERZERO, lo, optarg);
+        error_at_line(EXIT_FAILURE, 0, filename, lineno,
+                      FIXEDFORFILE" "NOTEMSG_LARGERZERO, lo, optarg);
       else
-	error(EXIT_FAILURE, 0, FIXEDFOROPTION" "NOTEMSG_LARGERZERO,
-	      lo, so, optarg);
+        error(EXIT_FAILURE, 0, FIXEDFOROPTION" "NOTEMSG_LARGERZERO,
+              lo, so, optarg);
     }
 }
 
@@ -459,11 +461,11 @@ gal_checkset_double_l_value(char *optarg, double *var, char *lo, char so,
   if(tmp<=value)
     {
       if(filename)
-	error_at_line(EXIT_FAILURE, 0, filename, lineno,
-		      FIXEDFORFILE" should be > %.4f", lo, optarg, value);
+        error_at_line(EXIT_FAILURE, 0, filename, lineno,
+                      FIXEDFORFILE" should be > %.4f", lo, optarg, value);
       else
-	error(EXIT_FAILURE, 0, FIXEDFOROPTION" should be > %.4f",
-	      lo, so, optarg, value);
+        error(EXIT_FAILURE, 0, FIXEDFOROPTION" should be > %.4f",
+              lo, so, optarg, value);
     }
 }
 
@@ -500,6 +502,54 @@ gal_checkset_any_double(char *optarg, double *var, char *lo, char so,
 
 
 
+/**************************************************************/
+/**********          Check fixed strings           ************/
+/**************************************************************/
+/* Check if the value to the `--type' option is recognized, if so set the
+   integer value. */
+void
+gal_checkset_known_types(char *optarg, int *bitpix, char *filename,
+                         size_t lineno)
+{
+  /* First check if the value is one of the accepted types. */
+  if     (strcmp(optarg, "byte")==0)     *bitpix=BYTE_IMG;
+  else if(strcmp(optarg, "short")==0)    *bitpix=SHORT_IMG;
+  else if(strcmp(optarg, "long")==0)     *bitpix=LONG_IMG;
+  else if(strcmp(optarg, "longlong")==0) *bitpix=LONGLONG_IMG;
+  else if(strcmp(optarg, "float")==0)    *bitpix=FLOAT_IMG;
+  else if(strcmp(optarg, "double")==0)   *bitpix=DOUBLE_IMG;
+  else
+    {
+      if(filename)
+        error_at_line(EXIT_FAILURE, 0, filename, lineno, "given value of "
+                      "the `type' option (`%s') is not recognized. It must "
+                      "be `byte', `short', `long', `longlong', `float', or "
+                      "`double'.", optarg);
+      else
+        error(EXIT_FAILURE, 0, "given value of the `--type' (`-T') option "
+              "(`%s') is not recognized. It must be `byte', `short', `long' "
+              "`longlong', `float', or `double'.", optarg);
+    }
+}
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 /**************************************************************/
 /**********          My String functions:          ************/
 /**************************************************************/
@@ -510,7 +560,7 @@ gal_checkset_string_has_space(char *in)
     switch(*in)
       {
       case ' ': case '\t': case '\v':
-	return 1;
+        return 1;
       }
   while(*(++in)!='\0');
   return 0;
@@ -609,10 +659,10 @@ gal_checkset_check_file(char *filename)
   FILE *tmpfile;
   errno=0;
   tmpfile = fopen(filename, "r");
-  if(tmpfile)			/* The file opened. */
+  if(tmpfile)                        /* The file opened. */
     {
       if(fclose(tmpfile)==EOF)
-	error(EXIT_FAILURE, errno, "%s", filename);
+        error(EXIT_FAILURE, errno, "%s", filename);
     }
   else
     error(EXIT_FAILURE, errno, "%s", filename);
@@ -622,6 +672,28 @@ gal_checkset_check_file(char *filename)
 
 
 
+/* Similar to `gal_checkset_check_file', but will report the result instead
+   of doing it quietly. */
+int
+gal_checkset_check_file_report(char *filename)
+{
+  FILE *tmpfile;
+  errno=0;
+  tmpfile = fopen(filename, "r");
+  if(tmpfile)                        /* The file opened. */
+    {
+      if(fclose(tmpfile)==EOF)
+        error(EXIT_FAILURE, errno, "%s", filename);
+      return 1;
+    }
+  else
+    return 0;
+}
+
+
+
+
+
 /* Check if a file exists. If so, remove it. */
 void
 gal_checkset_check_remove_file(char *filename, int dontdelete)
@@ -634,21 +706,21 @@ gal_checkset_check_remove_file(char *filename, int dontdelete)
      `r+`. */
   errno=0;
   tmpfile=fopen(filename, "r+");
-  if (tmpfile)			/* The file opened. */
+  if (tmpfile)                        /* The file opened. */
     {
       /* Close the file and make sure that it should be deleted. */
       errno=0;
       if(fclose(tmpfile))
-	error(EXIT_FAILURE, errno, "%s", filename);
+        error(EXIT_FAILURE, errno, "%s", filename);
       if(dontdelete)
-	error(EXIT_FAILURE, 0, "%s already exists and you have "
-	      "asked to not remove it with the `--dontdelete` "
-	      "(`-D`) option", filename);
+        error(EXIT_FAILURE, 0, "%s already exists and you have "
+              "asked to not remove it with the `--dontdelete` "
+              "(`-D`) option", filename);
 
       /* Delete the file: */
       errno=0;
       if(unlink(filename))
-	error(EXIT_FAILURE, errno, "%s", filename);
+        error(EXIT_FAILURE, errno, "%s", filename);
     }
   /* If the file doesn't exist, there is no problem, we wanted to
      remove it any way! Any other kind of error should not be
@@ -672,50 +744,50 @@ gal_checkset_dir_0_file_1(char *name, int dontdelete)
   struct stat nameinfo;
 
   if(name==NULL)
-    error(EXIT_FAILURE, 0, "a bug! The input to the gal_checkset_dir_0_file_1 "
-          "function in checkset.c should not be NULL. Please contact us at "
-          PACKAGE_BUGREPORT" so we can see what went wrong and fix it in "
-          "future updates");
+    error(EXIT_FAILURE, 0, "a bug! The input to the "
+          "gal_checkset_dir_0_file_1 function in checkset.c should not "
+          "be NULL. Please contact us at "PACKAGE_BUGREPORT" so we can "
+          "see what went wrong and fix it in future updates");
 
   errno=0;
   if(stat(name, &nameinfo)!=0)
     {
-      if(errno==ENOENT)	/* ENOENT: No such file or directory. */
-	{/* Make the file temporarily and see if everything is ok. */
-	  errno=0;
-	  tmpfile=fopen(name, "w");
-	  if (tmpfile)
-	    {
-	      fprintf(tmpfile, "Only to test write access.");
-	      errno=0;
-	      if(fclose(tmpfile))
-		error(EXIT_FAILURE, errno, "%s", name);
-	      errno=0;
-	      if(unlink(name))
-		error(EXIT_FAILURE, errno, "%s", name);
-	    }
-	  else
-	    error(EXIT_FAILURE, errno, "%s", name);
-	  return 1;		/* It is a file name, GOOD */
-	}
-      else			/* Some strange condition, ABORT */
-	error(EXIT_FAILURE, errno, "%s", name);
+      if(errno==ENOENT)        /* ENOENT: No such file or directory. */
+        {/* Make the file temporarily and see if everything is ok. */
+          errno=0;
+          tmpfile=fopen(name, "w");
+          if (tmpfile)
+            {
+              fprintf(tmpfile, "Only to test write access.");
+              errno=0;
+              if(fclose(tmpfile))
+                error(EXIT_FAILURE, errno, "%s", name);
+              errno=0;
+              if(unlink(name))
+                error(EXIT_FAILURE, errno, "%s", name);
+            }
+          else
+            error(EXIT_FAILURE, errno, "%s", name);
+          return 1;                    /* It is a file name, GOOD */
+        }
+      else                             /* Some strange condition, ABORT */
+        error(EXIT_FAILURE, errno, "%s", name);
     }
 
-  if(S_ISDIR(nameinfo.st_mode))	/* It is a directory, BAD */
+  if(S_ISDIR(nameinfo.st_mode))        /* It is a directory, BAD */
     return 0;
-  else if (S_ISREG(nameinfo.st_mode)) /* It is a file, GOOD. */
+  else if (S_ISREG(nameinfo.st_mode))  /* It is a file, GOOD. */
     {
       gal_checkset_check_remove_file(name, dontdelete);
       return 1;
     }
-  else 				/* Not a file or a dir, ABORT */
+  else                                 /* Not a file or a dir, ABORT */
     error(EXIT_FAILURE, 0, "%s not a file or a directory", name);
 
-  error(EXIT_FAILURE, 0, "a bug! In gal_checkset_dir_0_file_1, (in checkset.c) "
-	"The process should not reach the end of the function! Please "
-        "contact us at "PACKAGE_BUGREPORT" so we can see what went wrong "
-        "and fix it in future updates");
+  error(EXIT_FAILURE, 0, "a bug! In gal_checkset_dir_0_file_1, (in "
+        "checkset.c). The process should not reach the end of the "
+        "function! Please contact us at "PACKAGE_BUGREPORT" so we can "
+        "see what went wrong and fix it in future updates");
   return 0;
 }
 
@@ -728,33 +800,48 @@ gal_checkset_dir_0_file_1(char *name, int dontdelete)
    be removed and the given suffix will be put in the end. */
 void
 gal_checkset_automatic_output(char *inname, char *suffix, int removedirinfo,
-		int dontdelete, char **outname)
+                              int dontdelete, char **outname)
 {
   char *out;
   size_t i, l, offset=0;
 
-  /* Note that we are just using malloccat here to allocate the right
-     space! The contents of the allocated space will be changed after
-     this.*/
+  /* Merge the contents of the input name and suffix name (while also
+     allocating the necessary space).*/
   out=gal_checkset_malloc_cat(inname, suffix);
 
-  /* Put the input in the space and remove all elements including and after '.'.
-     Note that if there is no '.' in the name, gal_checkset_malloc_cat has
-     already appended inname and suffix.*/
-  l=strlen(inname);
-  strcpy(out, inname);
-  for(i=l;i!=0;--i)
+  /* If there is actually a suffix, replace it with the (possibly) existing
+     suffix. */
+  if(suffix[0]!='\0')
     {
-      /* We don't want to remove any '.' in a directory name so if a
-	 '/' is encountered in our search from the end of the file
-	 name, we won't continue. */
-      if(out[i]=='/') break;
-      else if(out[i]=='.')
-	{
-	  out[i]='\0';
-	  strcat(out, suffix);
-	  break;
-	}
+      /* Start from the end of the input array*/
+      l=strlen(inname);
+      for(i=l-1;i!=0;--i)
+        {
+          /* We don't want to touch anything before a `/' (directory
+             names). We are only concerned with file names here. */
+          if(out[i]=='/')
+            {
+              /* When `/' is the last input character, then the input is
+                 clearly not a filename, but a directory name. In this
+                 case, adding a suffix is meaningless (a suffix belongs to
+                 a filename for Gnuastro's tools). So close the string
+                 after the `/' and leave the loop. However, if the `/'
+                 isn't the last input name charector, there is probably a
+                 filename (without a "." suffix), so break from the
+                 loop. No further action is required, since we initially
+                 allocated the necessary space and concatenated the input
+                 and suffix arrays. */
+              if(i==l-1)
+                out[i+1]='\0';
+              break;
+            }
+          else if(out[i]=='.')
+            {
+              out[i]='\0';
+              strcat(out, suffix);
+              break;
+            }
+        }
     }
 
   /* If it is desired to remove the directory information from the
@@ -764,12 +851,12 @@ gal_checkset_automatic_output(char *inname, char *suffix, int removedirinfo,
   if(removedirinfo)
     {
       l=strlen(out);
-      for(i=l;i!=0;--i)	 	  /* Find the last forward slash.      */
-	if(out[i]=='/')
-	  {offset=i+1; break;}
+      for(i=l;i!=0;--i)         /* Find the last forward slash.      */
+        if(out[i]=='/')
+          {offset=i+1; break;}
       if(offset)
-	for(i=offset;i<=l;++i)	/* <= because we want to shift the   */
-	  out[i-offset]=out[i]; /* '\0' character in the string too. */
+        for(i=offset;i<=l;++i)  /* <= because we want to shift the   */
+          out[i-offset]=out[i]; /* '\0' character in the string too. */
     }
 
   /* Remove the created filename if it already exits. */
@@ -845,9 +932,9 @@ gal_checkset_not_dir_part(char *input)
   /* Get the length of the notdir name: */
   l=strlen(tmp);
   errno=0;
-  out=malloc(l*sizeof *out);
+  out=malloc((l+1)*sizeof *out);
   if(out==NULL)
-    error(EXIT_FAILURE, errno, "%lu bytes for notdir", l*sizeof *out);
+    error(EXIT_FAILURE, errno, "%lu bytes for notdir", (l+1)*sizeof *out);
 
   strcpy(out, tmp);
   return out;
@@ -877,24 +964,24 @@ gal_checkset_check_dir_write_add_slash(char **dirname)
   file_d=mkstemp(tmpname);
   if(file_d==-1)
     error(EXIT_FAILURE, errno, "cannot write output in the directory %s",
-	  indir);
+          indir);
   /*
   errno=0;
   printf("\n\n%s\n\n", tmpname);
   if( write(file_d, buf, strlen(buf)) == -1 )
     error(EXIT_FAILURE, errno, "%s: writing to this temporary file to "
-	  "check the given `%s` directory", tmpname, indir);
+          "check the given `%s` directory", tmpname, indir);
   */
   errno=0;
   if( close(file_d) == -1 )
     error(EXIT_FAILURE, errno, "%s: Closing this temporary file to check "
-	  "the given `%s` directory", tmpname, indir);
+          "the given `%s` directory", tmpname, indir);
 
   /* Delete the temporary file: */
   errno=0;
   if(unlink(tmpname)==-1)
     error(EXIT_FAILURE, errno, "%s: removing this temporary file made "
-	  "to check the given `%s directory`", tmpname, indir);
+          "to check the given `%s directory`", tmpname, indir);
 
   /* Remove the extra characters that were added for the random name. */
   tmpname[strlen(tmpname)-14]='\0';
