@@ -30,13 +30,12 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <stdlib.h>
 #include <string.h>
 
+#include <gnuastro/fits.h>
 #include <gnuastro/mesh.h>
-#include <gnuastro/mode.h>
 #include <gnuastro/qsort.h>
 #include <gnuastro/neighbors.h>
 #include <gnuastro/linkedlist.h>
 #include <gnuastro/statistics.h>
-#include <gnuastro/fitsarrayvv.h>
 #include <gnuastro/spatialconvolve.h>
 
 
@@ -83,45 +82,10 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 /*********************************************************************/
 /************       Finding the proper mesh ID      ******************/
 /*********************************************************************/
-/* There are two interal ways to store the IDs of the meshs:
-
-   1. By default, the mesh IDs are set based on their position within
-      the channels (so the meshs in each channel are contiguous). We
-      call this the channel-based mesh ID. The cgarray1 and cgarray2
-      store the mesh values based on this ID. All the basic mesh
-      properties, like their types and height and widths are stored
-      based on this ID.
-
-   2. If the user asks for a full image interpolation or smoothing,
-      then two new arrays are created called fgarray1 and
-      fgarray2. These arrays keep the mesh values as they appear on
-      the image, irrespective of which channel they belong to. We call
-      this the image-based ID. The image based ID is contiguous over
-      the image.
-
-  There is a third pointer called garray1 and garray2. These can point
-  to either the cgarrays or the fgarrays. Ideally, the user should be
-  completely ignorant to what garray is pointing to. The caller knows
-  the type of IDs they are using, but they don't know what to put into
-  garray. We call the ID that should be put into garray the
-  garray-based ID. It can be either of the two kinds above.
-
-  So we have made the following two functions:
-
-    gal_mesh_ch_based_id_from_gid:
-
-       This is useful for when you are going over the elements in
-       garray (and you are completley ignorant to which one of
-       cgarrays or fgarrays garray points to) and you need the channel
-       based IDs to get basic mesh information like the mesh type and
-       size.
-
-    gal_mesh_gid_from_ch_based_id:
-
-       This function is useful for the opposite case: you are going
-       over the meshs through the channel-based IDs, but you need to
-       know what ID to use for garray.
-*/
+/* This is useful for when you are going over the elements in garray (and
+you are completley ignorant to which one of cgarrays or fgarrays garray
+points to) and you need the channel based IDs to get basic mesh information
+like the mesh type and size. */
 size_t
 gal_mesh_ch_based_id_from_gid(struct gal_mesh_params *mp, size_t gid)
 {
@@ -273,9 +237,9 @@ gal_mesh_img_xy_to_mesh_id(struct gal_mesh_params *mp, size_t x, size_t y)
 /*********************************************************************/
 /* By default, the garray1 and garray2 arrays keep the meshes of each
    channel contiguously. So in practice, each channel is like a small
-   independent image. This will cause problems when we want to work on
-   the meshs irrespective of which channel they belong to. This
-   function allocates and fills in the fgarray1 and fgarray2 arrays.
+   independent image. This will cause problems when we want to work on the
+   meshs irrespective of which channel they belong to. This function
+   allocates and fills in the fgarray1 and fgarray2 arrays.
 
    The explanation above is for the case when reverse==0. If it is set
    equal to 1 (or any non-zero number), then
@@ -347,13 +311,13 @@ gal_mesh_full_garray(struct gal_mesh_params *mp, int reverse)
     }
 
   /* Just for a check:
-  gal_fitsarray_array_to_fits_img("nochannels.fits", "fgarray1", FLOAT_IMG, fgarray1,
-                 mp->nch2*mp->gs0, mp->nch1*mp->gs1, 1, NULL, NULL,
-                 "mesh");
+  gal_fits_array_to_file("nochannels.fits", "fgarray1", FLOAT_IMG,
+                         fgarray1, mp->nch2*mp->gs0, mp->nch1*mp->gs1, 1,
+                         NULL, NULL, "mesh");
   if(mp->ngarrays==2)
-    gal_fitsarray_array_to_fits_img("nochannels.fits", "fgarray2", FLOAT_IMG, fgarray2,
-                   mp->nch2*mp->gs0, mp->nch1*mp->gs1, 1, NULL, NULL,
-                   "mesh");
+    gal_fits_array_to_file("nochannels.fits", "fgarray2", FLOAT_IMG,
+                           fgarray2, mp->nch2*mp->gs0, mp->nch1*mp->gs1,
+                           1, NULL, NULL, "mesh");
   */
 }
 
@@ -380,7 +344,7 @@ gal_mesh_full_garray(struct gal_mesh_params *mp, int reverse)
 /*********************************************************************/
 /* Save the meshid of each pixel into an array the size of the image. */
 void
-gal_check_mesh_id(struct gal_mesh_params *mp, long **out)
+gal_mesh_check_mesh_id(struct gal_mesh_params *mp, long **out)
 {
   long i, *l, *lp;
   size_t row, start, *types=mp->types;
@@ -392,7 +356,8 @@ gal_check_mesh_id(struct gal_mesh_params *mp, long **out)
   errno=0;
   *out=calloc(mp->s0*mp->s1, sizeof **out);
   if(*out==NULL)
-    error(EXIT_FAILURE, errno, "the array to show mesh labels in checkmesh");
+    error(EXIT_FAILURE, errno,
+          "the array to show mesh labels in checkmesh");
 
   /* Fill the indexs: */
   for(i=0;i<mp->nmeshi;++i)
@@ -418,7 +383,8 @@ gal_check_mesh_id(struct gal_mesh_params *mp, long **out)
    input image. Note that the check arrays are only the size of the
    number of meshs, not the actual input image size. */
 void
-gal_mesh_check_garray(struct gal_mesh_params *mp, float **out1, float **out2)
+gal_mesh_check_garray(struct gal_mesh_params *mp, float **out1,
+                      float **out2)
 {
   int ngarrays=mp->ngarrays;
   size_t gid, row, start, chbasedid, *types=mp->types;
@@ -430,8 +396,9 @@ gal_mesh_check_garray(struct gal_mesh_params *mp, float **out1, float **out2)
      there is no overlap. */
   errno=0; *out1=malloc(mp->s0*mp->s1*sizeof **out1);
   if(*out1==NULL)
-    error(EXIT_FAILURE, errno, "%lu bytes for out1 in gal_mesh_check_garray "
-          "(mesh.c)", mp->s0*mp->s1*sizeof **out1);
+    error(EXIT_FAILURE, errno,
+          "%lu bytes for out1 in gal_mesh_check_garray (mesh.c)",
+          mp->s0*mp->s1*sizeof **out1);
   if(ngarrays==2)
     {
       errno=0; *out2=malloc(mp->s0*mp->s1*sizeof **out2);
@@ -481,8 +448,9 @@ gal_mesh_check_garray(struct gal_mesh_params *mp, float **out1, float **out2)
 
 /* Save the mesh grid values into an output file. */
 void
-gal_mesh_value_file(struct gal_mesh_params *mp, char *filename, char *extname1,
-                    char *extname2, struct wcsprm *wcs, char *spack_string)
+gal_mesh_value_file(struct gal_mesh_params *mp, char *filename,
+                    char *extname1, char *extname2, struct wcsprm *wcs,
+                    char *spack_string)
 {
   float *tmp1=NULL, *tmp2=NULL;
 
@@ -494,29 +462,29 @@ gal_mesh_value_file(struct gal_mesh_params *mp, char *filename, char *extname1,
          used for this job. In cgarray the meshs are ordered
          differently. */
       if(mp->garray1==mp->cgarray1) gal_mesh_full_garray(mp, 0);
-      gal_fitsarray_array_to_fits_img(filename, extname1, FLOAT_IMG,
-                                      mp->fgarray1, mp->gs0*mp->nch2,
-                                      mp->gs1*mp->nch1, 0, wcs, NULL,
-                                      spack_string);
+      gal_fits_array_to_file(filename, extname1, FLOAT_IMG,
+                             mp->fgarray1, mp->gs0*mp->nch2,
+                             mp->gs1*mp->nch1, 0, wcs, NULL,
+                             spack_string);
       if(mp->ngarrays==2)
-        /* Note that gal_mesh_full_garray will correct both the meshs if there
-           are two.*/
-        gal_fitsarray_array_to_fits_img(filename, extname2, FLOAT_IMG,
-                                        mp->fgarray2, mp->gs0*mp->nch2,
-                                        mp->gs1*mp->nch1, 0, wcs, NULL,
-                                        spack_string);
+        /* Note that gal_mesh_full_garray will correct both the meshs if
+           there are two.*/
+        gal_fits_array_to_file(filename, extname2, FLOAT_IMG,
+                               mp->fgarray2, mp->gs0*mp->nch2,
+                               mp->gs1*mp->nch1, 0, wcs, NULL,
+                               spack_string);
 
     }
   else
     {
       gal_mesh_check_garray(mp, &tmp1, &tmp2);
-      gal_fitsarray_array_to_fits_img(filename, extname1, FLOAT_IMG, tmp1,
-                                      mp->s0, mp->s1, 0, wcs, NULL,
-                                      spack_string);
+      gal_fits_array_to_file(filename, extname1, FLOAT_IMG, tmp1,
+                             mp->s0, mp->s1, 0, wcs, NULL,
+                             spack_string);
       if(mp->ngarrays==2)
-        gal_fitsarray_array_to_fits_img(filename, extname2, FLOAT_IMG, tmp2,
-                                        mp->s0, mp->s1, 0, wcs, NULL,
-                                        spack_string);
+        gal_fits_array_to_file(filename, extname2, FLOAT_IMG, tmp2,
+                               mp->s0, mp->s1, 0, wcs, NULL,
+                               spack_string);
       free(tmp1);
       free(tmp2);
     }
@@ -573,7 +541,7 @@ gal_mesh_value_file(struct gal_mesh_params *mp, char *filename, char *extname1,
    id of 1. In the actual function, the indexs are given in the order
    the types are found.
  */
-void
+static void
 fillmeshinfo(struct gal_mesh_params *mp, size_t chs0, size_t chs1,
              size_t lasts0, size_t lasts1)
 {
@@ -600,7 +568,7 @@ fillmeshinfo(struct gal_mesh_params *mp, size_t chs0, size_t chs1,
       if(ts1[typeind]>mp->maxs1) mp->maxs1=ts1[typeind];
 
       for(chi=0;chi<nch2;++chi)   /* Don't forget that C and FITS */
-	for(chj=0;chj<nch1;++chj) /* axises have different order. */
+        for(chj=0;chj<nch1;++chj) /* axises have different order. */
           {
             chid=chi*nch1+chj;
             for(i=0;i<lasti;++i)
@@ -630,7 +598,7 @@ fillmeshinfo(struct gal_mesh_params *mp, size_t chs0, size_t chs1,
       if(ts1[typeind]>mp->maxs1) mp->maxs1=ts1[typeind];
 
       for(chi=0;chi<nch2;++chi)
-	for(chj=0;chj<nch1;++chj)
+        for(chj=0;chj<nch1;++chj)
           {
             chid=chi*nch1+chj;
             for(j=0;j<lastj;++j)
@@ -659,7 +627,7 @@ fillmeshinfo(struct gal_mesh_params *mp, size_t chs0, size_t chs1,
       if(ts1[typeind]>mp->maxs1) mp->maxs1=ts1[typeind];
 
       for(chi=0;chi<nch2;++chi)
-	for(chj=0;chj<nch1;++chj)
+        for(chj=0;chj<nch1;++chj)
           {
             chid=chi*nch1+chj;
             for(i=0;i<lasti;++i)
@@ -688,24 +656,24 @@ fillmeshinfo(struct gal_mesh_params *mp, size_t chs0, size_t chs1,
       if(ts1[typeind]>mp->maxs1) mp->maxs1=ts1[typeind];
 
       for(chi=0;chi<nch2;++chi)
-	for(chj=0;chj<nch1;++chj)
-	  {
+        for(chj=0;chj<nch1;++chj)
+          {
             ++totalmeshcount;
-	    meshid=(chi*nch1+chj)*nmeshc + gs0*gs1-1;
-	    types[meshid]=typeind;
+            meshid=(chi*nch1+chj)*nmeshc + gs0*gs1-1;
+            types[meshid]=typeind;
             chindex[meshid]=gs0*gs1-1;
-	    imgindex[meshid]=( (chi*gs0+(gs0-1)) * nch1*gs1
+            imgindex[meshid]=( (chi*gs0+(gs0-1)) * nch1*gs1
                                + (chj*gs1+(gs1-1)) );
-	    start[meshid]=( (chi*chs0+(gs0-1)*meshsize) * s1
-				 + (chj*chs1+(gs1-1)*meshsize) );
-	  }
+            start[meshid]=( (chi*chs0+(gs0-1)*meshsize) * s1
+                                 + (chj*chs1+(gs1-1)*meshsize) );
+          }
     }
 
   /* Just for a check: */
   if(totalmeshcount!=mp->nmeshi)
-    error(EXIT_FAILURE, 0, "a bug! Please contact us at %s so we can fix it. "
-          "The basic information for some meshes has not been found (in "
-          "fillmeshinfo of mesh.c)", PACKAGE_BUGREPORT);
+    error(EXIT_FAILURE, 0, "a bug! Please contact us at %s so we can fix "
+          "it. The basic information for some meshes has not been found "
+          "(in fillmeshinfo of mesh.c)", PACKAGE_BUGREPORT);
 }
 
 
@@ -856,10 +824,10 @@ gal_mesh_free_mesh(struct gal_mesh_params *mp)
    1. A pointer to the gal_mesh_params structure that keeps all the
       information.
 
-   2. A pointer to a function that returns and gets a `void *' as its
-      only argument. This function will be directly given to
-      pthread_create. Through this function, you can any function that
-      you wish to operate on the mesh grid with.
+   2. A pointer to a function that returns and gets a `void *' as its only
+      argument. This function will be directly given to
+      pthread_create. Through this argument, you can choose the function to
+      operate on the mesh grid.
 
    3. The size of each element to copy the mesh grid into, this has to
       be type size of the same type that constitutes `img' in
@@ -869,11 +837,14 @@ gal_mesh_free_mesh(struct gal_mesh_params *mp)
       example sort them) in each mesh.
 
    4. If the value of this argument is 1, then a second garray will be
-      allocated in case your operation needs one.
+      allocated in case your operation needs it.
+
+   5. Wether the allocated garrays should be initialized or not.
 */
 void
-gal_mesh_operate_on_mesh(struct gal_mesh_params *mp, void *(*meshfunc)(void *),
-                         size_t oneforallsize, int makegarray2, int initialize)
+gal_mesh_operate_on_mesh(struct gal_mesh_params *mp,
+                         void *(*meshfunc)(void *), size_t oneforallsize,
+                         int makegarray2, int initialize)
 {
   int err;
   size_t i, nb;
@@ -949,22 +920,22 @@ gal_mesh_operate_on_mesh(struct gal_mesh_params *mp, void *(*meshfunc)(void *),
   else
     {
       /* Initialize the attributes. Note that this running thread
-	 (that spinns off the nt threads) is also a thread, so the
-	 number the barrier should be one more than the number of
-	 threads spinned off. */
+         (that spinns off the nt threads) is also a thread, so the
+         number the barrier should be one more than the number of
+         threads spinned off. */
       if(mp->nmeshi<numthreads) nb=mp->nmeshi+1;
       else                      nb=numthreads+1;
       gal_threads_attr_barrier_init(&attr, &mp->b, nb);
 
       /* Spin off the threads: */
       for(i=0;i<numthreads;++i)
-	if(mp->indexs[i*mp->thrdcols]!=GAL_THREADS_NON_THRD_INDEX)
-	  {
+        if(mp->indexs[i*mp->thrdcols]!=GAL_THREADS_NON_THRD_INDEX)
+          {
             mtp[i].id=i;
-	    mtp[i].mp=mp;
-	    err=pthread_create(&t, &attr, meshfunc, &mtp[i]);
-	    if(err) error(EXIT_FAILURE, 0, "can't create thread %lu", i);
-	  }
+            mtp[i].mp=mp;
+            err=pthread_create(&t, &attr, meshfunc, &mtp[i]);
+            if(err) error(EXIT_FAILURE, 0, "can't create thread %lu", i);
+          }
 
       /* Wait for all threads to finish and free the spaces. */
       pthread_barrier_wait(&mp->b);
@@ -998,7 +969,7 @@ gal_mesh_operate_on_mesh(struct gal_mesh_params *mp, void *(*meshfunc)(void *),
 /*********************************************************************/
 /********************         Interpolate         ********************/
 /*********************************************************************/
-void
+static void
 preparemeshinterparrays(struct gal_mesh_params *mp)
 {
   size_t bs0=mp->gs0, bs1=mp->gs1;
@@ -1085,7 +1056,7 @@ preparemeshinterparrays(struct gal_mesh_params *mp)
    the differences. But here we are not dealing with subpixel
    distances, so manhattan distance is fine. It also avoids the square
    root function which is much slower. */
-float
+static float
 manhattandistance(long ind, long xc, long yc, long s1)
 {
   return labs(ind%s1 - xc) + labs(ind/s1 - yc);
@@ -1095,9 +1066,9 @@ manhattandistance(long ind, long xc, long yc, long s1)
 
 
 /* Some of the variables have different names than the gal_mesh_params
-   structure because they are to be fed into the GAL_NEIGHBORS_FILL_4_ALLIMG
-   macro. */
-void *
+   structure because they are to be fed into the
+   GAL_NEIGHBORS_FILL_4_ALLIMG macro. */
+static void *
 meshinterponthread(void *inparams)
 {
   struct gal_mesh_thread_params *mtp=(struct gal_mesh_thread_params *)inparams;
@@ -1210,11 +1181,12 @@ meshinterponthread(void *inparams)
                   mp->errstart, currentnum, thisind/mp->nmeshc, mtp->id,
                   thisind);
         }
-      gal_linkedlist_tosll_free(lQ);  /* The rest of the queue not needed. */
+      gal_linkedlist_tosll_free(lQ);  /* Rest of the queue not needed. */
 
 
       /* Find the median of the nearest neighbors and put it in: */
-      qsort(nearest1, numnearest, sizeof *nearest1, gal_qsort_float_increasing);
+      qsort(nearest1, numnearest, sizeof *nearest1,
+            gal_qsort_float_increasing);
       outgarray1[thisind] = ( numnearest%2 ?
                               nearest1[numnearest/2] : /* Odd.  */
                               (nearest1[numnearest/2]  /* Even. */
@@ -1272,22 +1244,22 @@ gal_mesh_interpolate(struct gal_mesh_params *mp, char *errstart)
   else
     {
       /* Initialize the attributes. Note that this running thread
-	 (that spinns off the nt threads) is also a thread, so the
-	 number the barrier should be one more than the number of
-	 threads spinned off. */
+         (that spinns off the nt threads) is also a thread, so the
+         number the barrier should be one more than the number of
+         threads spinned off. */
       if(mp->nmeshi<numthreads) nb=mp->nmeshi+1;
       else                      nb=numthreads+1;
       gal_threads_attr_barrier_init(&attr, &mp->b, nb);
 
       /* Spin off the threads: */
       for(i=0;i<numthreads;++i)
-	if(mp->indexs[i*mp->thrdcols]!=GAL_THREADS_NON_THRD_INDEX)
-	  {
+        if(mp->indexs[i*mp->thrdcols]!=GAL_THREADS_NON_THRD_INDEX)
+          {
             mtp[i].id=i;
-	    mtp[i].mp=mp;
-	    err=pthread_create(&t, &attr, meshinterponthread, &mtp[i]);
-	    if(err) error(EXIT_FAILURE, 0, "can't create thread %lu", i);
-	  }
+            mtp[i].mp=mp;
+            err=pthread_create(&t, &attr, meshinterponthread, &mtp[i]);
+            if(err) error(EXIT_FAILURE, 0, "can't create thread %lu", i);
+          }
 
       /* Wait for all threads to finish and free the spaces. */
       pthread_barrier_wait(&mp->b);
@@ -1311,12 +1283,12 @@ gal_mesh_interpolate(struct gal_mesh_params *mp, char *errstart)
 
   /* For a check
   system("rm test.fits");
-  gal_fitsarray_array_to_fits_img("test.fits", "garray1", FLOAT_IMG, mp->garray1,
-                 mp->nch2*mp->gs0, mp->nch1*mp->gs1, 1, NULL, NULL,
-                 "mesh");
-  gal_fitsarray_array_to_fits_img("test.fits", "garray2", FLOAT_IMG, mp->garray2,
-                 mp->nch2*mp->gs0, mp->nch1*mp->gs1, 1, NULL, NULL,
-                 "mesh");
+  gal_fits_array_to_file("test.fits", "garray1", FLOAT_IMG, mp->garray1,
+                          mp->nch2*mp->gs0, mp->nch1*mp->gs1, 1, NULL, NULL,
+                          "mesh");
+  gal_fits_array_to_file("test.fits", "garray2", FLOAT_IMG, mp->garray2,
+                         mp->nch2*mp->gs0, mp->nch1*mp->gs1, 1, NULL, NULL,
+                         "mesh");
   */
 
   /* Clean up. */
@@ -1380,16 +1352,17 @@ gal_mesh_smooth(struct gal_mesh_params *mp)
 
       /* Do the spatial convolution */
       gal_spatialconvolve_convolve(mp->fgarray1, gs0*mp->nch2, gs1*mp->nch1,
-                                   kernel, smoothwidth, smoothwidth, numthreads,
-                                   1, &sgarray1);
+                                   kernel, smoothwidth, smoothwidth,
+                                   numthreads, 1, &sgarray1);
 
       free(mp->fgarray1);
       mp->garray1=mp->fgarray1=sgarray1;
       if(mp->ngarrays==2)
         {
-          gal_spatialconvolve_convolve(mp->fgarray2, gs0*mp->nch2, gs1*mp->nch1,
-                                       kernel, smoothwidth, smoothwidth,
-                                       mp->numthreads, 1, &sgarray2);
+          gal_spatialconvolve_convolve(mp->fgarray2, gs0*mp->nch2,
+                                       gs1*mp->nch1, kernel, smoothwidth,
+                                       smoothwidth, mp->numthreads,
+                                       1, &sgarray2);
           free(mp->fgarray2);
           mp->garray2=mp->fgarray2=sgarray2;
         }
@@ -1415,8 +1388,9 @@ gal_mesh_smooth(struct gal_mesh_params *mp)
         if(mp->ngarrays==2)
           {
             charray=&mp->cgarray2[chid*nmeshc];
-            gal_spatialconvolve_convolve(charray, gs0, gs1, kernel, smoothwidth,
-                                         smoothwidth, numthreads, 1, &tmp);
+            gal_spatialconvolve_convolve(charray, gs0, gs1, kernel,
+                                         smoothwidth, smoothwidth,
+                                         numthreads, 1, &tmp);
             o=tmp; fp=(f=charray)+gs0*gs1; do *f=*o++; while(++f<fp);
             free(tmp);
           }
@@ -1447,7 +1421,7 @@ gal_mesh_smooth(struct gal_mesh_params *mp)
 /*********************************************************************/
 /********************      Convolve in mesh       ********************/
 /*********************************************************************/
-void*
+static void*
 meshspatialconvonthreads(void *inparam)
 {
   struct gal_mesh_thread_params *mtp = (struct gal_mesh_thread_params *)inparam;
@@ -1483,7 +1457,7 @@ meshspatialconvonthreads(void *inparam)
       /* If the mesh is on the edge of the channel it should be
          treated differently compared to when it is not. */
       if( x>=xmin && ystart>=ymin && fx<xmax && fy<ymax )
-	{                       /* All pixels in this mesh are distant  */
+        {                       /* All pixels in this mesh are distant  */
           for(;x<fx;++x)        /* enough from the edge of the channel. */
             for(y=ystart;y<fy;++y)
               {
@@ -1502,12 +1476,12 @@ meshspatialconvonthreads(void *inparam)
                     conv[x*is1+y] = sum/ksum;
                   }
               }
-	}
+        }
       else
-	{                       /* Some pixels in this mesh are too  */
-	  for(;x<fx;++x)        /* close to the edge.                */
-	    for(y=ystart;y<fy;++y)
-	      {
+        {                       /* Some pixels in this mesh are too  */
+          for(;x<fx;++x)        /* close to the edge.                */
+            for(y=ystart;y<fy;++y)
+              {
                 if( isnan(img[x*is1+y]) )
                     conv[x*is1+y]=NAN;
                 else
@@ -1542,7 +1516,7 @@ meshspatialconvonthreads(void *inparam)
                     conv[x*is1+y] = sum/ksum;
                   }
             }
-	}
+        }
     }
 
   /* Free alltype and if multiple threads were used, wait until all
@@ -1625,24 +1599,24 @@ gal_mesh_spatial_convolve_on_mesh(struct gal_mesh_params *mp, float **conv)
   else
     {
       /* Initialize the attributes. Note that this running thread
-	 (that spinns off the nt threads) is also a thread, so the
-	 number the barrier should be one more than the number of
-	 threads spinned off. */
+         (that spinns off the nt threads) is also a thread, so the
+         number the barrier should be one more than the number of
+         threads spinned off. */
       if(mp->nmeshi<numthreads) nb=mp->nmeshi+1;
       else                      nb=numthreads+1;
       gal_threads_attr_barrier_init(&attr, &mp->b, nb);
 
       /* Spin off the threads: */
       for(i=0;i<numthreads;++i)
-	if(mp->indexs[i*mp->thrdcols]!=GAL_THREADS_NON_THRD_INDEX)
-	  {
+        if(mp->indexs[i*mp->thrdcols]!=GAL_THREADS_NON_THRD_INDEX)
+          {
             mtp[i].id=i;
-	    mtp[i].mp=mp;
+            mtp[i].mp=mp;
             mtp[i].conv=*conv;
             mtp[i].chbrd=chbrd;
-	    err=pthread_create(&t, &attr, meshspatialconvonthreads, &mtp[i]);
-	    if(err) error(EXIT_FAILURE, 0, "can't create thread %lu", i);
-	  }
+            err=pthread_create(&t, &attr, meshspatialconvonthreads, &mtp[i]);
+            if(err) error(EXIT_FAILURE, 0, "can't create thread %lu", i);
+          }
 
       /* Wait for all threads to finish and free the spaces. */
       pthread_barrier_wait(&mp->b);
@@ -1659,17 +1633,18 @@ gal_mesh_spatial_convolve_on_mesh(struct gal_mesh_params *mp, float **conv)
 
 
 
-/* The indexs array for correcting the convolution on inner channel edges has
-   been allocated.  Note that gal_threads_dist_in_threads will distribute indexs
-   from zero to numpix-1.  After it, we should fill in all the channels.
+/* The indexs array for correcting the convolution on inner channel edges
+   has been allocated.  Note that gal_threads_dist_in_threads will
+   distribute indexs from zero to numpix-1.  After it, we should fill in
+   all the channels.
 
    The method of filling in the indexs array with the proper indexs to
    re-convolve is very similar to the method explained below in
-   gal_mesh_change_to_full_convolution, where it is explained how to count the
-   number of pixels that should be re-convolved. */
-void
-corrconvindexs(struct gal_mesh_params *mp, size_t **indexs, size_t *numpix,
-               size_t *thrdcols)
+   gal_mesh_change_to_full_convolution, where it is explained how to count
+   the number of pixels that should be re-convolved. */
+static void
+corrconvindexs(struct gal_mesh_params *mp, size_t **indexs,
+               size_t *numpix, size_t *thrdcols)
 {
   size_t i, j, a, b;
   size_t numthreads=mp->numthreads;
@@ -1740,8 +1715,8 @@ corrconvindexs(struct gal_mesh_params *mp, size_t **indexs, size_t *numpix,
               ahigh = i*npch0<hk0    ? 0  : i*npch0-hk0;
 
               /* For a check:
-              printf("2: alow: %-4lu ahigh: %-4lu. b: %-10lu (%lu -- %lu)\n",
-                     alow, ahigh, b, blow, bhigh);
+              printf("2: alow: %-4lu ahigh: %-4lu. b: %-10lu "
+                     "(%lu -- %lu)\n", alow, ahigh, b, blow, bhigh);
               */
               for(a=alow;a<ahigh;++a)
                 {
@@ -1791,11 +1766,11 @@ corrconvindexs(struct gal_mesh_params *mp, size_t **indexs, size_t *numpix,
    (whose distance from the channel edges is more than half the PSF),
    do not need to undergo convolution again.
 
-   Note that the pixels on the edges of the image do not need to undergo this
-   correction.  Basically this function is very similar to
-   gal_spatialconvolve_convolve (spatialconvolve.c), other than the fact that
-   the indexs are not over the full image but only a select number of pixels.
-*/
+   Note that the pixels on the edges of the image do not need to undergo
+   this correction.  Basically this function is very similar to
+   gal_spatialconvolve_convolve (spatialconvolve.c), other than the fact
+   that the indexs are not over the full image but only a select number of
+   pixels. */
 void
 gal_mesh_change_to_full_convolution(struct gal_mesh_params *mp, float *conv)
 {
@@ -1819,8 +1794,8 @@ gal_mesh_change_to_full_convolution(struct gal_mesh_params *mp, float *conv)
   scp=malloc(mp->numthreads*sizeof *scp);
   if(scp==NULL)
     error(EXIT_FAILURE, errno,
-          "%lu bytes for scp in gal_mesh_change_to_full_convolution (mesh.c)",
-          mp->numthreads*sizeof *scp);
+          "%lu bytes for scp in gal_mesh_change_to_full_convolution "
+          "(mesh.c)", mp->numthreads*sizeof *scp);
 
 
   /* Put the indexs of the pixels to re-convolve here. */
@@ -1830,17 +1805,17 @@ gal_mesh_change_to_full_convolution(struct gal_mesh_params *mp, float *conv)
   /* Start the convolution on the desired pixels. */
   if(mp->numthreads==1)
     {
-      gal_spatialconvolve_pparams(mp->img, mp->s0, mp->s1, mp->kernel, mp->ks0,
-                                  mp->ks1, mp->numthreads, 1, conv, indexs,
-                                  &scp[0]);
+      gal_spatialconvolve_pparams(mp->img, mp->s0, mp->s1, mp->kernel,
+                                  mp->ks0, mp->ks1, mp->numthreads, 1,
+                                  conv, indexs, &scp[0]);
       gal_spatialconvolve_thread(&scp[0]);
     }
   else
     {
       /* Initialize the attributes. Note that this running thread
-	 (that spinns off the nt threads) is also a thread, so the
-	 number the barrier should be one more than the number of
-	 threads spinned off. */
+         (that spinns off the nt threads) is also a thread, so the
+         number the barrier should be one more than the number of
+         threads spinned off. */
       if(numpix<mp->numthreads) nb=numpix+1;
       else                      nb=mp->numthreads+1;
       gal_threads_attr_barrier_init(&attr, &b, nb);
@@ -1853,9 +1828,10 @@ gal_mesh_change_to_full_convolution(struct gal_mesh_params *mp, float *conv)
             gal_spatialconvolve_pparams(mp->img, mp->s0, mp->s1, mp->kernel,
                                         mp->ks0, mp->ks1, mp->numthreads, 1,
                                         conv, &indexs[i*thrdcols], &scp[i]);
-	    err=pthread_create(&t, &attr, gal_spatialconvolve_thread, &scp[i]);
-	    if(err)
-	      error(EXIT_FAILURE, 0, "can't create thread %lu", i);
+            err=pthread_create(&t, &attr, gal_spatialconvolve_thread,
+                               &scp[i]);
+            if(err)
+              error(EXIT_FAILURE, 0, "can't create thread %lu", i);
           }
 
       /* Wait for all threads to finish and free the spaces. */
