@@ -27,13 +27,14 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <errno.h>
 #include <error.h>
 #include <float.h>
+#include <stdint.h>
 #include <stdlib.h>
 
 #include <gnuastro/qsort.h>
+#include <gnuastro/array.h>
 #include <gnuastro/statistics.h>
-#include <gnuastro/arraymanip.h>
-
 
+#include "gnuastro/mode.h"
 
 
 
@@ -44,6 +45,46 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 /****************************************************************
  *****************    Mininum and Maximum    ********************
  ****************************************************************/
+/* Find the the minimum (non-blank) value in an array of type long. Note
+   that the long type doesn't have a NaN value like the floats above. So as
+   blank pixels, a value in the range of acceptable values is given. so we
+   have to explicitly ignore those values.*/
+void
+gal_statistics_long_non_blank_min(long *in, size_t size, long *min,
+                                  long blank)
+{
+  long tmin=INT32_MAX, *lpt;
+  lpt=in+size;
+  do
+    if(*in!=blank && *in<tmin)
+      tmin=*in;
+  while(++in<lpt);
+  *min=tmin;
+}
+
+
+
+
+
+/* Find the the minimum (non-blank) value in an array of type long. See the
+   explanation above `gal_statistics_long_min'. */
+void
+gal_statistics_long_non_blank_max(long *in, size_t size, long *max,
+                                  long blank)
+{
+  long tmax=INT32_MIN, *lpt;
+  lpt=in+size;
+  do
+    if(*in!=blank && *in>tmax)
+      tmax=*in;
+  while(++in<lpt);
+  *max=tmax;
+}
+
+
+
+
+
 void
 gal_statistics_float_min(float *in, size_t size, float *min)
 {
@@ -158,10 +199,10 @@ gal_statistics_float_second_max(float *in, size_t size, float *secondmax)
   do
     { /* Works for NAN, since NAN is not larger than any number. */
       if(*in>max)
-	{
-	  smax=max;
-	  max=*in;
-	}
+        {
+          smax=max;
+          max=*in;
+        }
       else if(*in>smax) smax=*in;
     }
   while(++in<fpt);
@@ -180,10 +221,10 @@ gal_statistics_float_second_min(float *in, size_t size, float *secondmin)
   do
     { /* Works for NAN, since NAN is not smaller than any number. */
       if(*in<min)
-	{
-	  smin=min;
-	  min=*in;
-	}
+        {
+          smin=min;
+          min=*in;
+        }
       else if(*in<smin) smin=*in;
     }
   while(++in<fpt);
@@ -260,8 +301,8 @@ gal_statistics_d_max_with_index(double *in, size_t size, double *max,
   do  /*  Works for NAN, see comments above. */
     if(*pt>tmax)
       {
-	tmax=*pt;
-	tindex=pt-in;
+        tmax=*pt;
+        tindex=pt-in;
       }
   while(++pt<fpt);
   *index=tindex;
@@ -283,8 +324,8 @@ gal_statistics_f_max_with_index(float *in, size_t size,
   do  /* Works for NAN, see comments above.x */
     if(*pt>tmax)
       {
-	tmax=*pt;
-	tindex=pt-in;
+        tmax=*pt;
+        tindex=pt-in;
       }
   while(++pt<fpt);
   *index=tindex;
@@ -306,8 +347,8 @@ gal_statistics_d_min_with_index(double *in, size_t size,
   do  /* Works for NAN, see comments above. */
     if(*pt<tmin)
       {
-	tmin=*pt;
-	tindex=pt-in;
+        tmin=*pt;
+        tindex=pt-in;
       }
   while(++pt<fpt);
   *index=tindex;
@@ -329,8 +370,8 @@ gal_statistics_f_min_with_index(float *in, size_t size,
   do /* Works for NAN, see comments above. */
     if(*pt<tmin)
       {
-	tmin=*pt;
-	tindex=pt-in;
+        tmin=*pt;
+        tindex=pt-in;
       }
   while(++pt<fpt);
   *index=tindex;
@@ -397,7 +438,7 @@ gal_statistics_float_sum_num(float *in, size_t *size)
 
 
 
-double
+static double
 doublesumnum(double *in, size_t *size)
 {
   double *fpt;
@@ -458,8 +499,8 @@ gal_statistics_float_sum_mask(float *in, unsigned char *mask,
   do
     if(*pt==0)
       {
-	sum+=in[pt-mask];
-	++counter;
+        sum+=in[pt-mask];
+        ++counter;
       }
   while(++pt<fpt);
 
@@ -483,8 +524,8 @@ gal_statistics_float_sum_mask_l(float *in, long *mask,
   do
     if(*pt==0)
       {
-	sum+=in[pt-mask];
-	++counter;
+        sum+=in[pt-mask];
+        ++counter;
       }
   while(++pt<fpt);
 
@@ -508,8 +549,8 @@ gal_statistics_float_sum_squared_mask(float *in, unsigned char *mask,
   do
     if(*pt==0)
       {
-	sum+=in[pt-mask] * in[pt-mask];
-	++counter;
+        sum+=in[pt-mask] * in[pt-mask];
+        ++counter;
       }
   while(++pt<fpt);
 
@@ -534,8 +575,8 @@ gal_statistics_float_sum_squared_mask_l(float *in, long *mask,
   do
     if(*pt==0)
       {
-	sum+=in[pt-mask] * in[pt-mask];
-	++counter;
+        sum+=in[pt-mask] * in[pt-mask];
+        ++counter;
       }
   while(++pt<fpt);
 
@@ -625,7 +666,7 @@ gal_statistics_f_ave_std(float *in, size_t size, float *ave,
       sum=gal_statistics_float_sum_mask(in, mask, size, &nsize1);
       sum2=gal_statistics_float_sum_squared_mask(in, mask, size, &nsize2);
       if(nsize1!=nsize2)
-	error(EXIT_FAILURE, 0, "a bug in gal_statistics_f_ave_std "
+        error(EXIT_FAILURE, 0, "a bug in gal_statistics_f_ave_std "
               "(lib/statistics.h).  Somehow the number of masked pixels is "
               "measured differently.  Please contact us so we can find the "
               "cause");
@@ -662,9 +703,9 @@ gal_statistics_f_ave_std_l(float *in, size_t size, float *ave,
       sum=gal_statistics_float_sum_mask_l(in, mask, size, &nsize1);
       sum2=gal_statistics_float_sum_squared_mask_l(in, mask, size, &nsize2);
       if(nsize1!=nsize2)
-	error(EXIT_FAILURE, 0, "a bug in favestl (lib/statistics.h). "
-	      "Somehow the number of masked pixels is measured "
-	      "differently. Please contact us so we can find the cause");
+        error(EXIT_FAILURE, 0, "a bug in favestl (lib/statistics.h). "
+              "Somehow the number of masked pixels is measured "
+              "differently. Please contact us so we can find the cause");
       size=nsize1;
     }
   *ave=sum/size;
@@ -675,45 +716,7 @@ gal_statistics_f_ave_std_l(float *in, size_t size, float *ave,
 
 
 
-/* Find the average and standard deviation of all pixels on a
-   region. A region in a larger image is defined by its starting pixel
-   (`start`), its height (s0) and width (s1). This function will find
-   the sum and sumsquared of all the nonmasked (==0 in mask[]) and non
-   marked (==0 in byt) pixels in the region. */
-void
-gal_statistics_f_ave_std_mask_byt_0_in_region(float *in, unsigned char *byt,
-                                              unsigned char *mask,
-                                              size_t startind, size_t s0,
-                                              size_t s1, size_t is1,
-                                              float *ave, float *std)
-{
-  float *i;
-  size_t r, size=0;
-  double sum=0, sumsq=0;
-  unsigned char *m, *b, *fb;
 
-  for(r=0;r<s0;++r)
-    {
-      i=in+startind;
-      m=mask+startind;
-      fb=(b=byt+startind)+s1;
-      do
-	{
-	if(*m++==0 && *b==0)	/* `m` will definitely be checked and */
-	  {			/* incremented, while `b` might not.  */
-	    size++;
-	    sum   += *i;
-	    sumsq += *i * *i;
-	  }
-	++i;
-	}
-      while(++b<fb);
-      startind+=is1;
-    }
-
-  *ave = sum/size;
-  *std = sqrt( (sumsq-sum*sum/size)/size );
-}
 
 
 
@@ -745,8 +748,8 @@ gal_statistics_median(float *array, size_t insize)
 
   /* Make a copy of the input, shift all its non-NaN elements to the
      start of the array, then sort them and find the median. */
-  gal_arraymanip_float_copy(array, insize, &copy);
-  gal_arraymanip_no_nans(copy, &size);
+  gal_array_float_copy(array, insize, &copy);
+  gal_array_no_nans(copy, &size);
   medind=gal_statistics_index_from_quantile(size, 0.5);
   qsort(copy, size, sizeof*copy, gal_qsort_float_increasing);
   median=copy[medind];
@@ -767,7 +770,7 @@ gal_statistics_median_double_in_place(double *array, size_t insize)
 
   /* Shift all its non-NaN elements to the start of the array, then
      sort them and find the median. */
-  gal_arraymanip_no_nans_double(array, &size);
+  gal_array_no_nans_double(array, &size);
 
   /* If all the elements are NaN (size==0), then return NaN,
      otherwise, find the median. */
@@ -834,15 +837,15 @@ gal_statistics_set_bins(float *sorted, size_t size, size_t numbins,
   if(min==max)
     {
       if(quant!=0.0f)
-	{
-	  min=sorted[ gal_statistics_index_from_quantile(size, quant)   ];
-	  max=sorted[ gal_statistics_index_from_quantile(size, 1-quant) ];
-	}
+        {
+          min=sorted[ gal_statistics_index_from_quantile(size, quant)   ];
+          max=sorted[ gal_statistics_index_from_quantile(size, 1-quant) ];
+        }
       else
-	{
-	  min=sorted[0];
-	  max=sorted[size-1];
-	}
+        {
+          min=sorted[0];
+          max=sorted[size-1];
+        }
     }
   binwidth=(max-min)/numbins;
 
@@ -924,10 +927,10 @@ gal_statistics_histogram(float *sorted, size_t size, float *bins,
   if(maxhistone)
     {
       for(i=0;i<numbins;++i)
-	if(bins[i*2+1]>max)
-	  max=bins[i*2+1];
+        if(bins[i*2+1]>max)
+          max=bins[i*2+1];
       for(i=0;i<numbins;++i)
-	bins[i*2+1]/=max;
+        bins[i*2+1]/=max;
     }
 
   /* In case you want to see the histogram:
@@ -1141,9 +1144,9 @@ gal_statistics_sigma_clip_converge(float *array, int o1_n0, size_t num_elem,
 
   if(o1_n0==0)
     {
-      gal_arraymanip_float_copy(array, num_elem, &orderedarray);
+      gal_array_float_copy(array, num_elem, &orderedarray);
       qsort(orderedarray, num_elem, sizeof*orderedarray,
-	    gal_qsort_float_increasing);
+            gal_qsort_float_increasing);
     }
   else orderedarray=array;
 
@@ -1160,33 +1163,33 @@ gal_statistics_sigma_clip_converge(float *array, int o1_n0, size_t num_elem,
                counter+1, med, ave, std, num_elem);
 
       /* It might happen that ave and std are NaN. If so, stop the
-	 process here (the user has not given a mask and some pixels
-	 have nan values!). */
+         process here (the user has not given a mask and some pixels
+         have nan values!). */
       if(isnan(ave) || isnan(std))
-	return 0;
+        return 0;
 
       /* Normally, oldstd should be larger than std, because the
-	 possible outliers have been removed. If it is not, it means
-	 that we have clipped too much! */
+         possible outliers have been removed. If it is not, it means
+         that we have clipped too much! */
       if(counter>0 && (oldstd-std)/std<accuracy)
-	{
-	  *outstd=oldstd; *outave=oldave; *outmed=oldmed;
-	  return 1;
-	}
+        {
+          *outstd=oldstd; *outave=oldave; *outmed=oldmed;
+          return 1;
+        }
 
       for(dpt=start; dpt<start+num_elem; ++dpt)
-	if (*dpt>med-sigma_multiple*std)
-	  {
-	    start=dpt;
-	    break;
-	  }
+        if (*dpt>med-sigma_multiple*std)
+          {
+            start=dpt;
+            break;
+          }
 
       for(dpt=oldstart+num_elem-1;dpt>start;dpt--)
-	if (*dpt<med+sigma_multiple*std)
-	  {
-	    num_elem=dpt-start+1;
-	    break;
-	  }
+        if (*dpt<med+sigma_multiple*std)
+          {
+            num_elem=dpt-start+1;
+            break;
+          }
 
       oldave=ave;
       oldmed=med;
@@ -1215,9 +1218,9 @@ gal_statistics_sigma_clip_certain_num(float *array, int o1_n0, size_t num_elem,
 
   if(o1_n0==0)
     {
-      gal_arraymanip_float_copy(array, num_elem, &orderedarray);
+      gal_array_float_copy(array, num_elem, &orderedarray);
       qsort(orderedarray, num_elem, sizeof*orderedarray,
-	    gal_qsort_float_increasing);
+            gal_qsort_float_increasing);
     }
   else orderedarray=array;
 
@@ -1234,26 +1237,26 @@ gal_statistics_sigma_clip_certain_num(float *array, int o1_n0, size_t num_elem,
                counter+1, med, ave, std, num_elem);
 
       /* It might happen that ave and std are nan. If so, stop the
-	 process here (the user has not given a mask and some pixels
-	 have nan values!). */
+         process here (the user has not given a mask and some pixels
+         have nan values!). */
       if(isnan(ave) || isnan(std))
-	return 0;
+        return 0;
 
 
       for(dpt=start; dpt<start+num_elem; ++dpt)
-	if (*dpt>med-sigma_multiple*std)
-	  {
-	    start=dpt;
-	    break;
-	  }
+        if (*dpt>med-sigma_multiple*std)
+          {
+            start=dpt;
+            break;
+          }
 
 
       for(dpt=oldstart+num_elem-1;dpt>start;dpt--)
-	if (*dpt<med+sigma_multiple*std)
-	  {
-	    num_elem=dpt-start+1;
-	    break;
-	  }
+        if (*dpt<med+sigma_multiple*std)
+          {
+            num_elem=dpt-start+1;
+            break;
+          }
     }
 
   if(o1_n0==0)
@@ -1326,10 +1329,10 @@ gal_statistics_remove_outliers_flat_cdf(float *sorted, size_t *outsize)
   for(i=maxind+1;i<size-1;++i)
     if(slopes[i]<minslope)
       {
-	if(firstfound)
-	  break;
-	else
-	  firstfound=1;
+        if(firstfound)
+          break;
+        else
+          firstfound=1;
       }
   *outsize=i+1;
 
@@ -1341,3 +1344,214 @@ gal_statistics_remove_outliers_flat_cdf(float *sorted, size_t *outsize)
 
   free(slopes);
 }
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+/****************************************************************/
+/*************          Mode calculation         ****************/
+/****************************************************************/
+void
+gal_statistics_mode_mirror_plots(float *sorted, size_t size, size_t mirrorindex,
+                                 float min, float max, size_t numbins,
+                                 char *histsname, char *cfpsname,
+                                 float mirrorplotdist)
+{
+  FILE *fp;
+  size_t i, msize;
+  float *out, maxhist=-FLT_MAX, maxcfp, d;
+  int normhist=0, maxhistone=0, normcfp=0;
+  float *bins, *mirror, *actual, mf, onebinvalue=0.0f;
+
+
+  /* Find the index of the mirror and make the mirror array: */
+  mf=sorted[mirrorindex];
+  gal_array_float_copy(sorted, size, &actual);
+  makemirrored(sorted, mirrorindex, &mirror, &msize);
+
+
+  /* Set the best range if asked for, such that the mirror is on the
+     1/3 of the image scale. */
+  if(mirrorplotdist!=0.0f)
+    {
+      min=actual[gal_statistics_index_from_quantile(size, 0.001f)];
+      max=mf+mirrorplotdist*(mf-min);
+    }
+
+
+  /* set the mirror pixel to have the value of zero.*/
+  min-=mf;
+  max-=mf;
+  gal_array_fsum_const(actual, size, -1.0f*mf);
+  gal_array_fsum_const(mirror, msize, -1.0f*mf);
+
+
+  /* Allocate space for the 3 column array keeping the histograms:*/
+  errno=0;
+  out=malloc(numbins*3*sizeof *out);
+  if(out==NULL)
+    error(EXIT_FAILURE, errno, "%lu bytes for out in "
+          "gal_mode_make_mirror_plots (mode.c)", numbins*3*sizeof *out);
+
+
+  /* Define the bin sides: */
+  gal_statistics_set_bins(actual, size, numbins, min,
+                          max, onebinvalue, 0, &bins);
+
+
+  /* Find the histogram of the actual data and put it in out. Note
+     that maxhistone=0, because here we want to use one value for both
+     histograms so they are comparable. */
+  gal_statistics_histogram(actual, size, bins, numbins,
+                           normhist, maxhistone);
+  for(i=0;i<numbins;++i)
+    if(bins[i*2+1]>maxhist) maxhist=bins[i*2+1];
+  for(i=0;i<numbins;++i)
+    { out[i*3]=bins[i*2]; out[i*3+1]=bins[i*2+1]/maxhist; bins[i*2+1]=0.0f;}
+  bins[i*2+1]=0.0f; /* bins[] actually has numbins+1 elements. */
+  d=(out[3]-out[0])/2;
+
+
+  /* Find the histogram of the mirrired distribution and put it in
+     out: */
+  gal_statistics_histogram(mirror, msize, bins, numbins, normhist,
+                           maxhistone);
+  for(i=0;i<numbins;++i)
+    { out[i*3+2]=bins[i*2+1]/maxhist; bins[i*2+1]=0.0f;}
+  bins[i*2+1]=0.0f; /* bins[] actually has numbins+1 elements. */
+
+
+  /* Print out the histogram: */
+  errno=0;
+  fp=fopen(histsname, "w");
+  if(fp==NULL)
+    error(EXIT_FAILURE, errno, "could not open file %s", histsname);
+  fprintf(fp, "# Histogram of actual and mirrored distributions.\n");
+  fprintf(fp, "# Column 0: Value in the middle of this bin.\n");
+  fprintf(fp, "# Column 1: Input data.\n");
+  fprintf(fp, "# Column 2: Mirror distribution.\n");
+  for(i=0;i<numbins;++i)
+    fprintf(fp, "%-25.6f%-25.6f%-25.6f\n", out[i*3]+d,
+            out[i*3+1], out[i*3+2]);
+  fclose(fp);
+
+
+
+
+  /* Find the cumulative frequency plots of the two distributions: */
+  gal_statistics_cumulative_fp(actual, size, bins, numbins, normcfp);
+  for(i=0;i<numbins;++i)
+    { out[i*3+1]=bins[i*2+1]; bins[i*2+1]=0.0f; }
+  bins[i*2+1]=0.0f; /* bins[] actually has numbins+1 elements. */
+  gal_statistics_cumulative_fp(mirror, msize, bins, numbins, normcfp);
+  for(i=0;i<numbins;++i)
+    { out[i*3+2]=bins[i*2+1]; bins[i*2+1]=0.0f;}
+  bins[i*2+1]=0.0f; /* bins[] actually has numbins+1 elements. */
+
+
+  /* Since the two cumultiave frequency plots have to be on scale, see
+     which one is larger and divided both CFPs by the size of the
+     larger one. Then print the CFPs. */
+  if(size>msize) maxcfp=size;
+  else maxcfp=msize;
+  errno=0;
+  fp=fopen(cfpsname, "w");
+  if(fp==NULL)
+    error(EXIT_FAILURE, errno, "could not open file %s", cfpsname);
+  fprintf(fp, "# Cumulative frequency plot (average index in bin) of\n"
+          "# Actual and mirrored distributions.\n");
+  fprintf(fp, "# Column 0: Value in the middle of this bin.\n");
+  fprintf(fp, "# Column 1: Actual data.\n");
+  fprintf(fp, "# Column 2: Mirror distribution.\n");
+  for(i=0;i<numbins;++i)
+    fprintf(fp, "%-25.6f%-25.6f%-25.6f\n", out[i*3],
+            out[i*3+1]/maxcfp, out[i*3+2]/maxcfp);
+  fclose(fp);
+
+
+  /* Clean up. */
+  free(out);
+  free(bins);
+  free(mirror);
+  free(actual);
+}
+
+
+
+
+
+/* It happens that you have the symmetricity and you want the flux
+   value at that point, this function will do that job. In practice,
+   it just finds bf from the equation to calculate symmetricity in
+   modesymmetricity. */
+float
+gal_statistics_mode_value_from_sym(float *sorted, size_t size,
+                                   size_t modeindex, float sym)
+{
+  float mf=sorted[modeindex];
+  float af=
+    sorted[gal_statistics_index_from_quantile(2*modeindex+1,
+                           GAL_STATISTICS_MODE_SYMMETRICITY_LOW_QUANT)];
+  return sym*(mf-af)+mf;
+}
+
+
+
+
+
+/* Find the quantile of the mode of a sorted distribution. The return
+   value is either 0 (not accurate) or 1 (accurate). Accuracy is
+   defined based on the difference between the maximum and minimum
+   maxdiffs that were found during the golden section search.
+
+   A good mode will have:
+
+   modequant=(float)(modeindex)/(float)size;
+   modesym>GAL_MODE_SYM_GOOD && modequant>GAL_MODE_LOW_QUANT_GOOD
+*/
+void
+gal_statistics_mode_index_in_sorted(float *sorted, size_t size, float errorstdm,
+                                    size_t *modeindex, float *modesym)
+{
+  struct gal_statistics_mode_params mp;
+
+  /* Initialize the gal_mode_params structure: */
+  mp.size=size;
+  mp.sorted=sorted;
+  mp.tolerance=0.01;
+  mp.numcheck=size/2;
+  mp.errorstdm=errorstdm;
+  if(mp.numcheck>1000)
+    mp.interval=mp.numcheck/1000;
+  else mp.interval=1;
+  mp.lowi  = gal_statistics_index_from_quantile(size,
+                                 GAL_STATISTICS_MODE_LOW_QUANTILE);
+  mp.highi = gal_statistics_index_from_quantile(size,
+                                 GAL_STATISTICS_MODE_HIGH_QUANTILE);
+  mp.midi  = (((float)mp.highi
+               + MODE_GOLDEN_RATIO*(float)mp.lowi)
+              /(1+MODE_GOLDEN_RATIO));
+  mp.midd  = mirrormaxdiff(mp.sorted, mp.size, mp.midi, mp.numcheck,
+                           mp.interval, mp.errorstdm);
+
+  /* Do the golden section search and find the resulting
+     symmetricity. */
+  *modeindex=modegoldenselection(&mp);
+  modesymmetricity(sorted, size, *modeindex, errorstdm, modesym);
+}
